@@ -9,6 +9,7 @@ use crate::error::error;
 use crate::schemas::gatekeeper;
 use crate::schemas::protocol as P;
 use crate::schemas::sturdy;
+use crate::trace;
 
 use futures::Sink;
 use futures::SinkExt;
@@ -95,7 +96,6 @@ pub struct TunnelRelay
     outbound_assertions: Map<P::Handle, Vec<Arc<WireSymbol>>>,
     membranes: Membranes,
     pending_outbound: Vec<P::TurnEvent<AnyValue>>,
-    self_entity: Arc<Ref<()>>,
     output: UnboundedSender<LoanedItem<Vec<u8>>>,
     output_text: bool,
 }
@@ -239,7 +239,6 @@ impl TunnelRelay {
                 next_export_oid: 0,
             },
             pending_outbound: Vec::new(),
-            self_entity: self_entity.clone(),
         };
         if let Some(ir) = initial_ref {
             tr.membranes.export_ref(ir).inc_ref();
@@ -248,8 +247,10 @@ impl TunnelRelay {
             |io| Arc::clone(&tr.membranes.import_oid(t, &tr_ref, io).inc_ref().obj));
         dump_membranes!(tr.membranes);
         *tr_ref.lock() = Some(tr);
-        t.linked_task(crate::name!("writer"), output_loop(o, output_rx));
-        t.linked_task(crate::name!("reader"), input_loop(t.facet.clone(), i, tr_ref));
+        t.linked_task(Some(AnyValue::symbol("writer")),
+                      output_loop(o, output_rx));
+        t.linked_task(Some(AnyValue::symbol("reader")),
+                      input_loop(t.trace_collector(), t.facet.clone(), i, tr_ref));
         t.state.add_exit_hook(&self_entity);
         result
     }
@@ -303,12 +304,17 @@ impl TunnelRelay {
     fn handle_inbound_packet(&mut self, t: &mut Activation, p: P::Packet<AnyValue>) -> ActorResult {
         tracing::debug!(packet = ?p, "-->");
         match p {
+            P::Packet::Extension(b) => {
+                let P::Extension { label, fields } = *b;
+                tracing::info!(?label, ?fields, "received Extension from peer");
+                Ok(())
+            }
             P::Packet::Error(b) => {
                 tracing::info!(message = ?b.message.clone(),
                                detail = ?b.detail.clone(),
                                "received Error from peer");
                 Err(*b)
-            },
+            }
             P::Packet::Turn(b) => {
                 let P::Turn(events) = *b;
                 for P::TurnEvent { oid, event } in events {
@@ -393,8 +399,7 @@ impl TunnelRelay {
                         }
                     }
                 }
-                t.deliver();
-                Ok(())
+                t.commit()
             }
         }
     }
@@ -473,7 +478,19 @@ impl TunnelRelay {
 
     pub fn send_event(&mut self, t: &mut Activation, remote_oid: sturdy::Oid, event: P::Event<AnyValue>) -> ActorResult {
         if self.pending_outbound.is_empty() {
-            t.message_for_myself(&self.self_entity, ());
+            let self_ref = Arc::clone(&self.self_ref);
+            t.pre_commit(move |t| {
+                let mut g = self_ref.lock();
+                let tr = g.as_mut().expect("initialized");
+                let events = std::mem::take(&mut tr.pending_outbound);
+                tr.send_packet(&t.account(),
+                               events.len(),
+                               P::Packet::Turn(Box::new(P::Turn(events.clone()))))?;
+                for P::TurnEvent { oid, event } in events.into_iter() {
+                    tr.outbound_event_bookkeeping(t, sturdy::Oid(oid.0), &event)?;
+                }
+                Ok(())
+            });
         }
         self.pending_outbound.push(P::TurnEvent { oid: P::Oid(remote_oid.0), event });
         Ok(())
@@ -614,11 +631,13 @@ impl DomainEncode<Arc<Cap>> for Membranes {
 }
 
 async fn input_loop(
+    trace_collector: Option<trace::TraceCollector>,
     facet: FacetRef,
     i: Input,
     relay: TunnelRelayRef,
 ) -> Result<LinkedTaskTermination, Error> {
-    let account = Account::new(crate::name!("input-loop"));
+    let account = Account::new(Some(AnyValue::symbol("input-loop")), trace_collector);
+    let cause = trace::TurnCause::external("input-loop");
     match i {
         Input::Packets(mut src) => {
             loop {
@@ -626,12 +645,15 @@ async fn input_loop(
                 match src.next().await {
                     None => break,
                     Some(bs) => {
-                        let is_alive = facet.activate(Arc::clone(&account), |t| {
-                            let mut g = relay.lock();
-                            let tr = g.as_mut().expect("initialized");
-                            tr.handle_inbound_datagram(t, &bs?)
-                        });
-                        if !is_alive { break; }
+                        if !facet.activate(
+                            &account, Some(cause.clone()), |t| {
+                                let mut g = relay.lock();
+                                let tr = g.as_mut().expect("initialized");
+                                tr.handle_inbound_datagram(t, &bs?)
+                            })
+                        {
+                            break;
+                        }
                     }
                 }
             }
@@ -654,12 +676,15 @@ async fn input_loop(
                 match n {
                     0 => break,
                     _ => {
-                        let is_alive = facet.activate(Arc::clone(&account), |t| {
-                            let mut g = relay.lock();
-                            let tr = g.as_mut().expect("initialized");
-                            tr.handle_inbound_stream(t, &mut buf)
-                        });
-                        if !is_alive { break; }
+                        if !facet.activate(
+                            &account, Some(cause.clone()), |t| {
+                                let mut g = relay.lock();
+                                let tr = g.as_mut().expect("initialized");
+                                tr.handle_inbound_stream(t, &mut buf)
+                            })
+                        {
+                            break;
+                        }
                     }
                 }
             }
@@ -690,25 +715,15 @@ async fn output_loop(
 }
 
 impl Entity<()> for TunnelRefEntity {
-    fn message(&mut self, t: &mut Activation, _m: ()) -> ActorResult {
-        let mut g = self.relay_ref.lock();
-        let tr = g.as_mut().expect("initialized");
-        let events = std::mem::take(&mut tr.pending_outbound);
-        tr.send_packet(&t.account(), events.len(), P::Packet::Turn(Box::new(P::Turn(events.clone()))))?;
-        for P::TurnEvent { oid, event } in events.into_iter() {
-            tr.outbound_event_bookkeeping(t, sturdy::Oid(oid.0), &event)?;
-        }
-        Ok(())
-    }
-
-    fn exit_hook(&mut self, t: &mut Activation, exit_status: &Arc<ActorResult>) -> ActorResult {
+    fn exit_hook(&mut self, t: &mut Activation, exit_status: &Arc<ActorResult>) {
         if let Err(e) = &**exit_status {
             let e = e.clone();
             let mut g = self.relay_ref.lock();
             let tr = g.as_mut().expect("initialized");
-            tr.send_packet(&t.account(), 1, P::Packet::Error(Box::new(e)))?;
+            if let Err(f) = tr.send_packet(&t.account(), 1, P::Packet::Error(Box::new(e))) {
+                tracing::error!("Failed to send error packet: {:?}", f);
+            }
         }
-        Ok(())
     }
 }
 
